/*
Title: Draw with Light
Description: On Arduino, play a piezo with light (frequency of tone) and potentiometer (duration of tone)
 On Processing, draw a radar with light (radius of radar) and potentiometer (hue of radar). Use the 
 button to invert spin (clockwise or counterclockwise).
Components: photoresistor, potentiometer, piezo, button. 
Circuit:
 - photoresistor: one terminal to 5V, one terminal to GND with 10K resistor and to analog pin A0 
 - potentiometer: one side terminal to 5V, one side terminal to GND, central terminal to analog pin A1
 - piezo: one terminal to GND, one terminal to digital pin 8
 - button: one terminal to 5V, one terminal to GND with 10K resistor and to digital pin 2
*/

import processing.serial.*;
import java.util.Calendar;


Serial port;  
float angle = 0.0;
float radius = 0.0; 
int spin = 0;
int oldSpin = 0;
boolean clockwise = true;
int hue = 0;

int[] serialInArray = new int[3];    // Where we'll put what we receive
int serialCount = 0;                 // A count of how many bytes we receive
boolean firstContact = false;        // Whether we've heard from the microcontroller

int portNum = 5; // Arduino port number on my computer

void setup() {
  size(540, 540);
  frameRate(30);
  strokeWeight(4);
  smooth();
  // open serial port
  port = new Serial(this, Serial.list()[portNum], 9600);
  // empty serial buffer
  port.clear();
  colorMode(HSB, 255);
  background(255);
}

void draw() {
  if (radius > 0) {  // If data is available
    int middleX = width/2;
    int middleY = height/2;

    // white line cleans previous values
    float x = middleX + cos(angle) * height * 0.49;
    float y = middleY + sin(angle) * height * 0.49;
    stroke(255);
    line(middleX, middleY, x, y);

    // colored line shows current values
    x = middleX + cos(angle) * radius;
    y = middleY + sin(angle) * radius;
    stroke(hue, 255, 255);
    line(middleX, middleY, x, y);

    // if the buttons had been pressed, invert spin
    if (oldSpin == 0 && spin == 1) {
      clockwise = !clockwise;
    }
    // update button state
    oldSpin = spin;
    
    if (clockwise) {
      angle += 0.01;
    } else {
      angle -= 0.01;
    }
  }
}

void serialEvent(Serial port) {
  // read a byte from the serial port:
  int inByte = port.read();
  // if this is the first byte received, and it's an A,
  // clear the serial buffer and note that you've
  // had first contact from the microcontroller.
  // Otherwise, add the incoming byte to the array:
  if (firstContact == false) {
    if (inByte == 'A') {
      port.clear();          // clear the serial port buffer
      firstContact = true;     // you've had first contact from the microcontroller
      port.write('A');       // ask for more
    }
  } else {
    // Add the latest byte from the serial port to array:
    serialInArray[serialCount] = inByte;
    serialCount++;

    // If we have 3 bytes:
    if (serialCount > 2) {
      radius = map(serialInArray[0], 0, 255, 0, height * 0.48);
      hue = serialInArray[1];
      spin = serialInArray[2];

      // print the values (for debugging purposes only):
      //println(radius + "\t" + hue + "\t" + spin);


      // Send a capital A to request new sensor readings:
      port.write('A');
      // Reset serialCount:
      serialCount = 0;
    }
  }
}


// safe frame is 's' is pressed
void keyReleased() {  
  if (key == 's' || key == 'S') saveFrame(timestamp()+"_####.png");
}

// get the current timestamp
String timestamp() {
  Calendar now = Calendar.getInstance();
  return String.format("%1$ty%1$tm%1$td_%1$tH%1$tM%1$tS", now);
}