// Code from http://www.generative-gestaltung.de/code

/**
 * simple example of a recursive function
 * 
 * KEYS
 * 1-9                 : recursion level
 * s                   : save png
 */

int recursionLevel = 6;
float startRadius = 200;

void setup() {
  size(800, 600);
  smooth();
  noFill();
  strokeCap(PROJECT);
}

void draw() {

  background(255);
  
  translate(width/2, height/2);
  // bottom-up
  rotate(PI);
  
  drawBranch(0, 0, startRadius, recursionLevel);


}

void drawBranch(float x, float y, float radius, int level) {

  // as long as level is greater than zero, draw branch
  if (level > 0) {
    // draw arc
    strokeWeight(level*2);
    stroke(0, 130, 164, 100);
    noFill();
    arc(x, y, radius*2, radius*2, -PI, 0);
  
    // draw center dot
    fill(0);
    noStroke();
    ellipse(x, y, level*1.5, level*1.5);

    // draw left branch
    drawBranch(x-radius, y+radius/2, radius/2, level-1);

    // draw right branch
    drawBranch(x+radius, y+radius/2, radius/2, level-1);
  }
}



void keyReleased() {  
  
  // save image
  if (key == 's' || key == 'S') saveFrame("branching.png");
  
  // select level
  int k = int(key) - 48;
  if (k>=0 && k<=9) {
    recursionLevel = k;
  }
}