Node tree;
int max_fanout = 3; // maximum node fanout; must be >= 0
int max_level = 4; // maximum tree depth; must be >= 1

void setup() {
  fullScreen();
  background(255);
  tree = new Node(width/2, height/2, 1, max_fanout, 0, 0);
}

void draw() {
  tree.picture();
  tree.update();
}

void mouseReleased() {
  background(255);
  tree = new Node(width/2, height/2, 1, max_fanout, 0, 0);
} 

void keyPressed() {
  if (key=='s' || key=='S') {
    saveFrame("tree4.png");
  }
}



class Node {
  float x, y; // node screen coordinates
  float angle, radius; // node polar coordinates wrt parent node
  int level; // node level
  int fanout; // node fanout
  float rotation; // node rotation speed and direction
  Node[] children = {}; // list of children

  Node(float _x, float _y, int _level, int _fanout, float _angle, float _radius) {
    x = _x;
    y = _y;
    level = _level;
    fanout = _fanout;
    angle = _angle;
    radius = _radius;
    rotation = radians(random(-1, 1));     

    if (level < max_level) { // create children
      children = new Node[fanout];
      for (int i = 0; i < children.length; i++) {
        int clevel = level + 1;
        int cfanout;
        if (level < (max_level - 1)) { // node with internal children
          cfanout = max_fanout;
        } else { // node with leaf children
          cfanout = 0;
        }
        float cradius = random(50, 200);
        float cangle = random(0, TWO_PI);
        float cx = x + cos(cangle) * cradius;
        float cy = y + sin(cangle) * cradius;
        children[i] = new Node(cx, cy, clevel, cfanout, cangle, cradius);
      }
    }
  }


  void picture() {
    if (level < max_level) { // draw edges
      for (int i = 0; i < children.length; i++) { 
        color c = 0;
        switch(level) {
        case 1: 
          c = color(255, 0, 0); // red
          break;
        case 2: 
          c = color(0, 255, 0); // green
          break;
        case 3:
          c = color(0, 0, 255); // blue
          break;
        }
        stroke(c, 10);
        strokeWeight(5 / level);
        line(x, y, children[i].x, children[i].y); 
        children[i].picture();
      }
    }
  }

  void update() {
    for (int i = 0; i < children.length; i++) {
      children[i].angle += children[i].rotation;
      children[i].x = x + cos(children[i].angle) * children[i].radius;
      children[i].y = y + sin(children[i].angle) * children[i].radius;
      children[i].update();
    }
  }
}